@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python2.5
 
 import sys
 import os
@@ -12,17 +12,115 @@ except:
 try:
 	import gtk
 	import gtk.glade
+	import gobject
 except:
 	sys.exit(1)
 
 import hildon
 import osso
+import conic
+import dbus
+import dbus.glib
 
-#import gconf
 from gnome import gconf
+import threading
 
+#Class for running rdesktop-cli threads
+class rdpThread(threading.Thread) :
+	def __init__(self, name, host, parameters):
+		self.parameters = parameters
+		self.host = host
+		self.name = name
+		self.online = False #Assume online, just in case...
+		self.connection = conic.Connection()
+		self.checkedConn = False
+		self.connection.connect("connection-event", self.conn_event, 0x1234)
+		assert(self.connection.request_connection(conic.CONNECT_FLAG_NONE))
+		threading.Thread.__init__ ( self, name="test" )
+	def conn_event(self, connection, event, magic):
+		status = event.get_status()
+		if status == conic.STATUS_CONNECTED:
+			#print "online!\n"
+			self.online = True
+		elif status == conic.STATUS_DISCONNECTED:
+			#print "offline...\n"
+			self.online = False
+		self.checkedConn = True
+	def run(self):
+		while self.checkedConn == False:
+			time.sleep(0.1)
+		if self.online == False:
+			gtk.gdk.threads_enter()
+			self.ErrPopUp("Sorry, could not connect to the internet.\nConnection to \""+self.name+"\" failed.")
+			gtk.gdk.threads_leave();
+			return False
+		#print "rdesktop-cli " + self.parameters + " " + self.host
+		result = os.system("rdesktop-cli " + self.parameters + " " + self.host)
+		needEnter = False
+		for thread in threading.enumerate():
+			if thread.getName() == "gtk":
+				needEnter = True
+		if needEnter == True :
+			gtk.gdk.threads_enter()
+		if result == 512 : #This isn't actually an error, just a connection closed weird...
+			print "" #We just need something here...
+		elif result == 256 : #Connection refused, either no internet or bad host
+			#self.ErrPopUp("Sorry, there was an error connecting to "+self.name+".\nConnection refused.")
+			if self.online == True :
+				tryagain = self.TimedPopUp("Sorry, there was an error connecting to \""+self.name+"\".\nConnection refused.  Will try again in",6)
+				if tryagain == True :
+					rdpThread(self.name, self.host, self.parameters).start()
+			else :
+				self.ErrPopUp("Sorry, there was an error connecting to "+self.name+".\nConnection refused.")
+		elif result == 0 : #Exited fine
+			print "" #We just need something here...
+		elif result == 2 : #User control-Ced it...  So, they should know they did it, do nothing
+			print ""
+		else : #Some unknown error, tell them the error code (to report back, hopefully), and exit.
+			#self.ErrPopUp("Sorry, there was an error in the remote desktop connection with \""+self.name+"\".\nError code: "+str(result))
+			if self.online == True :
+				tryagain = self.TimedPopUp("Sorry, there was an error in the remote desktop connection with \""+self.name+"\".\nError code: "+str(result)+".  Will try again in",6)
+				if tryagain == True :
+					rdpThread(self.name, self.host, self.parameters).start()
+			else :
+				self.ErrPopUp("Sorry, there was an error in the remote desktop connection with \""+self.name+"\".\nError code: "+str(result))
+		#self.TimedPopUp("Test in", 5)
+		if needEnter == True :
+			gtk.gdk.threads_leave()
+		#return True
+	def ErrPopUp(self,msg):
+		error_dialog = gtk.MessageDialog(None,0,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE,msg)
+		def close(dialog, response, editor):
+			editor.about_dialog = None
+			dialog.destroy()
+		error_dialog.connect("response", close, self)
+		error_dialog.run()
+	def TimedPopUp(self, msg, secs) :
+		#OK -- -5, Cancel -- -6
+		self.msg = msg
+		self.secs = secs
+		self.return_val = False
+		self.error_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_ERROR, gtk.BUTTONS_OK_CANCEL, self.msg + " " + str(self.secs) + " seconds.")
+		def close(dialog, response, editor) :
+			if response == -5 :
+				editor.return_val = True
+			editor.about_dialog = None
+			dialog.destroy()
+		self.error_dialog.connect("response", close, self)
+		def update_dlg(parent) :
+			parent.secs = parent.secs - 1
+			if parent.secs == 0 :
+				parent.error_dialog.response(-5)
+				return False #to kill the loop
+			else :
+				parent.error_dialog.set_markup(parent.msg + " " + str(parent.secs) + " seconds.")
+				return True # Keep the loop going...
+		gobject.timeout_add(1000, update_dlg, self)
+		self.error_dialog.run()
+		return self.return_val
+		
 class rdesktop:
-	
+
 	def __init__(self):
 		if os.path.isfile("/usr/local/lib/rdesktop.glade") :
 			self.gladefile = "/usr/local/lib/rdesktop.glade"  
@@ -32,8 +130,8 @@ class rdesktop:
 		self.wTree = gtk.glade.XML(self.gladefile)
 		self.window = self.wTree.get_widget("wnd_main")
 		if (self.window):
-			self.window.connect("destroy", gtk.main_quit)
-		
+			self.window.connect("destroy", self.cancel)
+
 		self.name = self.wTree.get_widget("entry_name")
 		self.host = self.wTree.get_widget("entry_host")
 		self.username = self.wTree.get_widget("entry_username")
@@ -46,7 +144,7 @@ class rdesktop:
 		self.recent = self.wTree.get_widget("lst_recent")
 		self.toolbar = self.wTree.get_widget("toolbar")
 		self.mainMenu = self.wTree.get_widget("menu_main")
-		
+
 		#Advanced Preferences Dialog
 		self.pref_console = self.wTree.get_widget("pref_console")
 		self.dlg_advanced = self.wTree.get_widget("dlg_advanced")
@@ -55,23 +153,23 @@ class rdesktop:
 		self.pref_encryption = self.wTree.get_widget("pref_encryption")
 		self.pref_cancel = self.wTree.get_widget("pref_cancel")
 		self.pref_save = self.wTree.get_widget("pref_save")
-		
+
 		#About dialog
 		self.dlg_about = self.wTree.get_widget("dlg_about")
-		
+
 		#Hooray hildon!
 		self.app = hildon.Program()
 		self.window.set_title("gui")
 		self.app.add_window(self.window)
-		
+
 		#Redo the menu from the glade file...
 		menu = gtk.Menu()
 		for child in self.mainMenu.get_children():
 			child.reparent(menu)
 		self.window.set_menu(menu)
-			
+
 		self.mainMenu.destroy()
-				
+
 		#Get the profile list set up...
 		self.column = gtk.TreeViewColumn("recent", gtk.CellRendererText(), text=0)
 		self.profname = gtk.TreeViewColumn("profname", gtk.CellRendererText(), text=1)	
@@ -81,16 +179,16 @@ class rdesktop:
 		self.recent.set_model(self.recentList)
 		self.profname.set_visible(False)
 		#We load the profiles down a bit lower, once GConf is already set up
-		
+
 		#Make sure these have the first value selected (not no value)		
 		self.colordepth.set_active(0)
 		self.sound.set_active(0)
-		
+
 		#Hooray hildon! :-P  Don't autocapitalize host, username, password, and don't predict for password
 		self.host.set_property('hildon-input-mode', (1 << 0 | 1 << 1 | 1 << 2))
 		self.username.set_property('hildon-input-mode', (1 << 0 | 1 << 1 | 1 << 2))
 		self.password.set_property('hildon-input-mode', (1 << 0 | 1 << 1 | 1 << 2 | 1 << 29))
-		
+
 		#Setup GConf and load profiles
 		self.client = gconf.client_get_default()
 		if self.client.dir_exists("/apps/rdesktop") == True :
@@ -98,13 +196,16 @@ class rdesktop:
 			self.dirs = self.client.all_dirs("/apps/rdesktop")
 			for profile in self.dirs :
 				self.recentList.append([self.client.get_string(profile+"/name"),profile])
+			load_prof = self.client.get_string("/apps/rdesktop/load_prof")
+			if load_prof != None :
+				self.load_profile("", "", "", "/apps/rdesktop/prof"+load_prof)
 		else :
 			#No preferences?  Just make the /apps/rdesktop dir, and everything should work fine
 			self.client.add_dir("/apps/rdesktop", gconf.CLIENT_PRELOAD_NONE)
 
 		self.about_dialog = None  #-- Not sure if I need these two
 		self.error_dialog = None  #-- lines...
-		
+
 		#Keep track of which profile is selected -- This will be the number as a string ex: "1"
 		self.current_profile = ""
 
@@ -123,8 +224,7 @@ class rdesktop:
 			"save_advanced" : self.ok_advanced,
 			"cancel_advanced" : self.cancel_advanced}
 		self.wTree.signal_autoconnect(dic)
-			
-	
+
 	def ErrPopUp(self,msg):
 		error_dialog = gtk.MessageDialog(None,0,gtk.MESSAGE_ERROR,gtk.BUTTONS_CLOSE,msg)
 		def close(dialog, response, editor):
@@ -134,17 +234,17 @@ class rdesktop:
 		# error_dialog.connect("delete-event", delete_event, self)
 		self.error_dialog = error_dialog
 		error_dialog.run()
-	
+
 	def ok_advanced(self, widget) :
 		self.dlg_advanced.hide()
-		
+
 	def cancel_advanced(self, widget) :
 		#self.pref_console.set_active(self._tempconsole)
 		self.pref_compression.set_active(self._tempcompress)
 		self.pref_bitmap.set_active(self._tempbitmap)
 		self.pref_encryption.set_active(self._tempencryption)
 		self.dlg_advanced.hide()
-	
+
 	def show_extra(self,widget):
 		#self.dlg_advanced.show()
 		#self._tempconsole = self.pref_console.get_active()
@@ -156,11 +256,11 @@ class rdesktop:
 			self.ok_advanced("")
 		else :
 			self.cancel_advanced("")
-		
+
 	def show_about(self, widget):
 		response = self.dlg_about.run()
 		self.dlg_about.hide()
-	
+
 	def save_profile(self,widget):
 		if self.current_profile == "" :
 			#This is a new profile!  We'll name the profiles with numbers (prof1, prof2, ...)...
@@ -199,43 +299,42 @@ class rdesktop:
 				self.dirs = self.client.all_dirs("/apps/rdesktop")
 				for profile in self.dirs :
 					self.recentList.append([self.client.get_string(profile+"/name"),profile])
-				
-		
+
 	def connect(self,widget):
 		#Fullscreen, keep window manager keybindings, and username
 		to_run = "-f"
-		
+
 		#Username
 		if self.username.get_text() != "" :
-			to_run = to_run + " -u " + self.username.get_text()
-		
+			to_run = to_run + " -u \"" + self.username.get_text()+"\""
+
 		#Domain
 		if self.domain.get_text() != "" :
-			to_run = to_run + " -d " + self.domain.get_text()
-			
+			to_run = to_run + " -d \"" + self.domain.get_text()+"\""
+
 		#Password
 		if self.password.get_text() != "" :
-			to_run = to_run + " -p " + self.password.get_text()
-			
+			to_run = to_run + " -p \"" + self.password.get_text()+"\""
+
 		#Give it a good title...
 		to_run = to_run + " -T \""+self.name.get_text()+"\""
-		
+
 		# Share media cards?
 		if self.sharemem.get_active() == True :
 			to_run = to_run + " -r disk:mmc1=/media/mmc1,mmc2=/media/mmc2"
-			
+
 		#Share MyDocs?
 		if self.sharedocs.get_active() == True :
 			to_run = to_run + " -r disk:MyDocs=/home/user/MyDocs"
-			
+
 		#RDP Compression?
 		if self.pref_compression.get_active() == True :
 			to_run = to_run + " -z"
-			
+
 		#RDP Encryption?
 		if self.pref_encryption.get_active() == False :
 			to_run = to_run + " -E"
-			
+
 		#Color depth handling
 		if self.colordepth.get_active() == 0 :
 			to_run = to_run + " -a 8"
@@ -245,11 +344,11 @@ class rdesktop:
 			to_run = to_run + " -a 16"
 		else :
 			to_run = to_run + " -a 24"
-			
+
 		#Persistant bitmap caching
 		if self.pref_bitmap.get_active() == True :
 			to_run = to_run + " -P"
-			
+
 		# Sound handling
 		if self.sound.get_active() == 0 :
 			to_run = to_run + " -r sound:off"
@@ -257,32 +356,25 @@ class rdesktop:
 			to_run = to_run + " -r sound:local"
 		else :
 			to_run = to_run + " -r sound:remote"
-		
+
 		#Console mode? -- I wasn't sure how this worked, but appearently it's required sometimes...
 		if self.pref_console.get_active() == True :
 			to_run = to_run + " -0"
-		
-		#Now run it! -- I'd like to be able to keep the GUI open, but I need to figure out threading, first...
-		self.window.hide()
-		while gtk.events_pending():
-			gtk.main_iteration()
 
-		result = os.system("rdesktop-cli " + to_run + " " + self.host.get_text())
-		#print result;
-		if result == 512 : #This isn't actually an error, just a connection closed weird...
-			sys.exit(1);
-		elif result == 256 : #Connection refused, either no internet or bad host, so show GUI again
-			self.ErrPopUp("Sorry, there was an error in the remote desktop connection.\nConnection refused.")
-			self.window.show()
-		elif result == 0 : #Exited fine
-			sys.exit(1)
-		else : #Some unknown error, tell them the error code (to report back, hopefully), and exit.
-			self.ErrPopUp("Sorry, there was an error in the remote desktop connection.\nError code: "+result)
-			sys.exit(1)
-		
+		#Now run it! -- I'd like to be able to keep the GUI open, but I need to figure out threading, first...
+		#self.window.hide()
+		#while gtk.events_pending():
+		#	gtk.main_iteration()
+
+		rdpThread(self.name.get_text(), self.host.get_text(), to_run).start()
+		#Because we don't store the thread anywhere, we now loose control of the thread...
+		# But that's ok, because we don't need to do anything with it!
+
 	def cancel(self,widget) :
-		gtk.main_quit(self.window)
-		
+		if self.current_profile != "" :
+			self.client.set_string("/apps/rdesktop/load_prof", self.current_profile)
+		gtk.main_quit()
+
 	def save(self,prof) :
 		self.client.set_string("/apps/rdesktop/"+prof+"/name", self.name.get_text())
 		self.client.set_string("/apps/rdesktop/"+prof+"/server", self.host.get_text())
@@ -337,9 +429,10 @@ class rdesktop:
 		for profile in self.dirs :
 			self.recentList.append([self.client.get_string(profile+"/name"),profile])
 
-	def load_profile(self, widget, path, view_column) :
+	def load_profile(self, widget, path, view_column, profile="") :
 		#selection = self.recent.get_selection()
-		profile = str(self.recentList.get_value(self.recentList.get_iter(path),1))
+		if path != "" :
+			profile = str(self.recentList.get_value(self.recentList.get_iter(path),1))
 		self.name.set_text(self.client.get_string(profile + "/name"))
 		self.host.set_text(self.client.get_string(profile + "/server"))
 		self.username.set_text(self.client.get_string(profile + "/username"))
@@ -358,10 +451,19 @@ class rdesktop:
 		#self.ErrPopUp("selected prof"+num)
 		self.current_profile = num
 
+def startGTK() :
+	gtk.main()
+	#sys.exit(1)
 
 if __name__ == "__main__":
 	gtk.set_application_name("rdesktop")
 	title = 'rdesktop'
 	handle = rdesktop()
-	gtk.main()
-	sys.exit(1)
+	#bus = dbus.SystemBus(private=True)
+	gtk.gdk.threads_init()
+	threading.Thread(target=startGTK,name="gtk").start()
+	#while threading.activeCount() > 1:
+		#for thread in threading.enumerate():
+			#print thread.getName()
+	#	time.sleep(1)
+	#sys.exit(1)
